﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MotorBoat
{
    public partial class FormMapWithSetBoats : Form
    {
        private MapWithSetBoatsGeneric<DrawningObjectBoat, AbstractMap> _mapShipCollectionGeneric;
        private readonly Dictionary<string, AbstractMap> _mapDict = new()
        {
            {"Простая карта", new SimpleMap() },
            {"Розовая карта", new PinkMap() },
            {"Морская карта",new SeaMap() }
        };
        private readonly MapsCollection _mapsCollection;
        public FormMapWithSetBoats()
        {
            InitializeComponent();
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }

        private void ReloadMaps()
        {
            int index = ListBoxMaps.SelectedIndex;

            ListBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                string? list = _mapsCollection.Keys[i];
                ListBoxMaps.Items.Add(list);
            }

            if (ListBoxMaps.Items.Count > 0 && (index == -1 || index >= ListBoxMaps.Items.Count))
            {
                ListBoxMaps.SelectedIndex = 0;
            }
            else if (ListBoxMaps.Items.Count > 0 && index > -1 && index < ListBoxMaps.Items.Count)
            {
                ListBoxMaps.SelectedIndex = index;
            }
        }

        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender,
      EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Простая карта":
                    map = new SimpleMap();
                    break;
                case "Розовая карта":
                    map = new PinkMap();
                    break;
                case "Морская карта":
                    map = new SeaMap();
                    break;
            }
            if (map != null)
            {
                _mapShipCollectionGeneric = new MapWithSetBoatsGeneric<DrawningObjectBoat, AbstractMap>(
                    pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapShipCollectionGeneric = null;
            }
        }
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            if (ListBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            FormBoat form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                DrawningObjectBoat boat = new(form.SelectedBoat);
                if (_mapsCollection[ListBoxMaps.SelectedItem?.ToString() ??
               string.Empty] + boat !=-1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image =
                   _mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }

        }
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (ListBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (ListBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (ListBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (ListBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty].MoveObject(dir);
        }

        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!_mapDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
        }
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[ListBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }

        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (ListBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить карту {ListBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(ListBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
            }
        }
    }
    
}
